<?php
require_once __DIR__ . '/vendor/autoload.php';


use Jose\Component\Core\AlgorithmManager;
use Jose\Component\Encryption\Algorithm\KeyEncryption\A256KW;
use Jose\Component\Encryption\Algorithm\ContentEncryption\A256CBCHS512;
use Jose\Component\Encryption\Algorithm\ContentEncryption\A128CBCHS256;
use Jose\Component\Encryption\Compression\CompressionMethodManager;
use Jose\Component\Encryption\Compression\Deflate;
use Jose\Component\Encryption\JWEBuilder;
use Jose\Component\Encryption\JWEDecrypter;
use Jose\Component\Encryption\Serializer\JWESerializerManager;
use Jose\Component\Encryption\Serializer\CompactSerializer;
use Jose\Component\KeyManagement\JWKFactory;


// The key encryption algorithm manager with the A256KW algorithm.
$keyEncryptionAlgorithmManager = new AlgorithmManager([
    new A256KW(),
]);

// The content encryption algorithm manager with the A256CBC-HS256 algorithm.
$contentEncryptionAlgorithmManager = new AlgorithmManager([
    new A128CBCHS256(),
]);

// The compression method manager with the DEF (Deflate) method.
$compressionMethodManager = new CompressionMethodManager([
    new Deflate(),
]);

// We instantiate our JWE Builder.
$jweBuilder = new JWEBuilder(
    $keyEncryptionAlgorithmManager,
    $contentEncryptionAlgorithmManager,
    $compressionMethodManager
);

use Jose\Component\Core\JWK;

  /** COMMOM CODE (Creates JWK Key from shared string) */

/**
 * Utility function that converts a given shared key into JWK Key. 
 * @return JWK
 */
function createKey()
{
   $sharedKey = hex2bin('2457bf1dc01659e2cd0af14dced615376aaab76f40c0e89d641421f807112b84');
   

    $jwk = JWKFactory::createFromSecret(
        $sharedKey,
        // The shared secret
        [
            'alg' => 'A256KW',
            'use' => 'enc'
        ]
    );

    return $jwk;
}
function decrypt($encrypted_token) : string
{

     $jweDecrypter = new JWEDecrypter(
         $GLOBALS['keyEncryptionAlgorithmManager'],
         $GLOBALS['contentEncryptionAlgorithmManager'],
         $GLOBALS['compressionMethodManager']
     );

     $serializerManager = new JWESerializerManager([
         new CompactSerializer(),
     ]);

     $jwe = $serializerManager->unserialize($encrypted_token);

     $success = $jweDecrypter->decryptUsingKey($jwe, createKey(), 0);

     if ($success) {
         echo "\nSuccessfully Decrypted";
     } else {
         echo "Error while decrypting";
     }

     $plaintext = $jwe->getPayload();

     echo "\nOutput: \n" . $plaintext ."\n";

     return $plaintext;
}



function sign($message) : string {
     $privateKey =
openssl_pkey_get_private(file_get_contents('private.pem'));

     openssl_sign($message, $signature, $privateKey,
"sha256WithRSAEncryption");

     $encoded = base64_encode($signature);
     return $encoded;
}

/**
  *
  * Verifies if the given signature is indeed valid.
  *
  * @param mixed $message Original message for which the signature is
created.
  * @param mixed $messageSignature Signature itself
  * @return void
  */
function verify($message, $messageSignature): bool{

     $signature = base64_decode($messageSignature);

     $publicKey =
openssl_pkey_get_public(file_get_contents("public.pem"));

     $result = openssl_verify($message, $signature, $publicKey,
"sha256WithRSAEncryption");

     if ($result) {
         echo "\nSignature is valid";
         return true;
     } else {
         echo "\nSignature is not valid";
         return false;
     }
}



// The payload we want to encrypt. It MUST be a string.
// $payload = '{
//                 "Authorization": "Basic U1lFREFQSUFVVEg6MmE4OGE5MWE5MmE2NGEx",
//                 "txnPassword": "2a88a91a92a64a1a1a3",
//                 "srcAcctNumber": "2774201000198",
//                 "destAcctNumber": "9833111000032",
//                 "customerID": "13961989",
//                 "txnAmount": "1",
//                 "benefName": "test"
//             }';

$payloadneft = '{
  "Request": {
    "body": {
      "srcAccountDetails": {
        "currency": "INR",
        "branchCode": "3555",
        "branchName": "MEERA BAGH"
      },
      "destAccountDetails": {
        "ifscCode": "SBIN0009112",
        "benefBankName": "STATE BANK OF INDIA"
      },
      "txnCurrency": "INR",
      "purpose": "joocla",
      "encryptData": {
        "Authorization": "Basic 310593572:97a4a41a91a67a10a5a14a0 ",
        "txnPassword": "119a0a25a11a0a90a17a7a7a87a3a48",
        "srcAcctNumber": "120001344764",
        "destAcctNumber": "38791842321",
        "customerID": "310593572",
        "benefName": "Dipesh",
        "txnAmount": "11",
        "userRefNo": "payon3873"
      }
    }
  }
}';


$payload1 = '{
    "Request": {
        "body": {
            "srcAccountDetails": {
                "identity": "B001",
                "currency": "INR",
                "branchCode": "3555"
            },
            "destAccountDetails": {
                "identity": "B001",
                "currency": "INR"
            },
            "txnCurrency": "INR",
            "ifscCode": "SBIN0009112",
            "narration": "SIT NEFT ON ENABLED APP",
            "paymentMode": "N",
            "valueDate": "18-05-2022",
            "standingInstDetails": {
                "frequency": "1",
                "endOfPeriod": "N",
                "benefAcctType": "10",
                "mobileNo": "9538779182",
                "emailId": "ksdeepa@canarabank.com"
            },
      "encryptData": {
        "Authorization": "Basic UEFZT05BUEk6MTEzYTBhNTFhOGE0MWE4OWE3MmE4NGE3MWE4YTZhMTA=",
        "txnPassword": "102a8a34a3a44a88a72a84a71a8a6a10",
        "srcAcctNumber": "120001344764",
        "destAcctNumber": "38791842321",
        "customerID": "310593572",
        "benefName": "Dipesh",
        "txnAmount": "10",
        "userRefNo": "payon3873"
      }
    }
  }
}';

$payloadimps = '{
  "Request": {
    "body": {
      "srcAccountDetails": {
        "currency": "INR",
        "branchCode": "3555",
        "branchName": "MEERA BAGH"
      },
      "destAccountDetails": {
        "ifscCode": "SBIN0009112",
        "benefBankName": "STATE BANK OF INDIA"
      },
      "txnCurrency": "INR",
      "purpose": "joocla",
        "encryptData": {
          "Authorization": "Basic UEFZT05BUEk6MTEzYTBhNTFhOGE0MWE4OWE3MmE4NGE3MWE4YTZhMTA=",
          "txnPassword": "102a8a34a3a44a88a72a84a71a8a6a10",
          "srcAcctNumber": "120001344764",
          "destAcctNumber": "38791842321",
          "customerID": "310593572",
          "benefName": "Dipesh",
          "txnAmount": "10",
          "userRefNo": "payon92138"
        }
      }
    }
  }';

  $signatureimps = '{
    "Authorization": "Basic UEFZT05BUEk6MTEzYTBhNTFhOGE0MWE4OWE3MmE4NGE3MWE4YTZhMTA=",
    "txnPassword": "102a8a34a3a44a88a72a84a71a8a6a10",
    "srcAcctNumber": "120001344764",
    "destAcctNumber": "38791842321",
    "customerID": "310593572",
    "benefName": "Dipesh",
    "txnAmount": "10",
    "userRefNo": "payon92138"
  }';

  $signatureneft = '{
    "Authorization": "Basic UEFZT05BUEk6MTEzYTBhNTFhOGE0MWE4OWE3MmE4NGE3MWE4YTZhMTA=",
    "txnPassword": "102a8a34a3a44a88a72a84a71a8a6a10",
    "srcAcctNumber": "120001344764",
    "destAcctNumber": "38791842321",
    "customerID": "310593572",
    "benefName": "Dipesh",
    "txnAmount": "10",
    "userRefNo": "payon3873"
  }';

echo "<pre>";
// print_r($payload);
// echo "<br>";
// echo "<br>";




$payload = json_encode(json_decode($signatureimps));


$message = '{
        "Authorization": "Basic MzEwNTkzNTcyOjk3YTRhNDFhOTFhNjdhMTBhNWExNGEw",
        "txnPassword": "2a88a91a92a64a1a1a3",
        "srcAcctNumber": "120001344764",
        "destAcctNumber": "38791842321",
        "customerID": "13961989",
        "txnAmount": "11",
        "benefName": "Estella"
      }';

      //$message = json_encode(json_decode($message));

/** ENCRYPTION  CODE */

/**
 * Encrypts a given payload and returns the enc
 rypted token
 * @param mixed $payload Input to be encrypted
 * @return string Encrypted token
 */

 $teststring = '{
    "Authorization": "Basic cGF5b25hcGk6MmE4OGE5MWE5MmE2NGEx",
    "txnPassword": "2a88a91a92a64a1a1a3",
    "srcAcctNumber": "1228201003092",
    "destAcctNumber": "38791842321",
    "customerID": "13961989",
    "txnAmount": "11",
    "benefName": "Estella"
  }';
function encrypt($payload) : string
{
    $jwe = $GLOBALS['jweBuilder']
        ->create() // We want to create a new JWE
        ->withPayload($payload) // We set the payload
        ->withSharedProtectedHeader([
            'alg' => 'A256KW',
            // Key Encryption Algorithm
            'enc' => 'A128CBC-HS256',
            // Content Encryption Algorithm
            'zip' => 'DEF' // We enable the compression (irrelevant as the payload is small, just for the example).
        ])
        ->addRecipient(createKey()) // We add a recipient (a shared key or public key).
        ->build(); // We build it


    $serializer = new CompactSerializer(); // The serializer

    $token = $serializer->serialize($jwe, 0); // We serialize the recipient at index 0 (we only have one recipient).

   // print("Encrypted string: \n");

    //print($token);
    
    return $token;
}

// print_r($payload);
// echo "<br>";

$decr_str = "eyJlbmMiOiJBMTI4Q0JDLUhTMjU2IiwiYWxnIjoiQTI1NktXIn0.c83QT2dvVYBco0RCGlbMTVHEgAXbB6jlki13pXV0QNab8hNuC_k3MA.Dua1lhzfoE5-JfUx_sWsJg.cx6_w3YBnUM_1BRPooqqVqM9r_WVk0MLhbg5AxMVJvQfAQTDOAVPBL4mL95j0WXREM_NofZpmb4BiLA34GSGcflj5nbhHsC1B97LcG16rmBN0-mt277TVMXlFENklE-Z8W__3Flx_N6_BbMYUsYBeA.0aVHYgX35fxWbXz7RNk6Hg";

// $dec = decrypt($decr_str);
//exit;
// echo $dec;
// exit;
$signature = sign($payload);
// echo $signature;
// exit;


$result = encrypt($payload);
// echo "result : " . $result;
// echo "<br>";
// echo "<br>";
// exit;





/** call encrypt function */
//  print(encrypt($message));
// //  echo $enc;
//  //print_r($enc);

// exit;

 $encrypted_token = 'eyJlbmMiOiJBMTI4Q0JDLUhTMjU2IiwiYWxnIjoiQTI1NktXIn0.eBPNI_mQV60jtvX88wI6EFle1etR8vO8c7BdgYKJi65egzbo3wjbZA.sr0VxpXyIplICt7ovx6lZQ._1QjM_OZ4TKHC0SAYZLgGOfJikQmDLRZNJZ0q7pgNWfDtwI44rSl5Zqet_eCgjKNlVdCKnTQ3E58I-3cIVkmDZtt37KKaf4VbZDd3jaYkzor_Hn7hX2ZdeNytIEo7jILGjfMSKahog6jBM9TkhFDJlqDxNZKM-NFc1ziRtI7Ysi3D9OW1rv5_IAjA_hGKsUKvHYc5sZ5pEkFJiQWIRoUXhitejZxm_r8nt8qB306N4N5bcin6LGc5W3lpetnY8NtC396b1Xoz3C9CoBxEATPn_ImTfafdiz3LypobP2B08qfuDQtuqiX3XoWbUE1xdE-S4nj6OjNHfyd6flULQ5yeQ.ikcOTO357aB0tPOPMRr8YQ';

// echo "<br>";
// echo "<br>";

// /** call decrypt function */
echo "<pre>";
decrypt($encrypted_token);
exit;

// echo "<br>";
// echo "<br>";

// print(sign($payload));
// // exit;
// echo "<br>";
// echo "<br>";
verify($payloadimps, 'eyJlbmMiOiJBMTI4Q0JDLUhTMjU2IiwiYWxnIjoiQTI1NktXIn0.AM2DMI6AfNe682xwsQCT51WjJfMwP81AEw5pnftIEIveQjLbM8hU6A.gXJRJicaUDCZHxHdH0d2LA.t0bjccMdkd0aB0F7lwB8ltJVnfXhTq0nXchCl8ngtB2C4ZQxSChSuZMf2xr-UKzVt1DvKuMQBpw7S18ukEUaI6ScbT5q5brUEahFmxcxBlE.gqWpMzMlB39fyOww4aos-w');